#ifndef _OPENDRIVE_OBJECT_H
#define _OPENDRIVE_OBJECT_H

#include "opendrive_common.h"
#include "opendrive_road_segment.h"
#ifdef _HAVE_XSD
#include "../src/xml/OpenDRIVE_1.4H.hxx"
#endif

#define OD_MAX_VERTICES     32

typedef enum {OD_NONE,OD_BOX,OD_CYLINDER,OD_POLYGON} object_type_t;

typedef struct
{
  double length;
  double width;
  double height;
}TOpendriveBox;

typedef struct
{
  double radius;
  double height;
}TOpendriveCylinder;

typedef struct
{  
  TOpendriveTrackPoint vertices[OD_MAX_VERTICES];
  double height[OD_MAX_VERTICES];
  unsigned int num_vertices;
}TOpendrivePolygon;

typedef union
{
  TOpendriveBox box;
  TOpendriveCylinder cylinder;
  TOpendrivePolygon polygon;
}TOpendriveObject;

class COpendriveRoadSegment;

class COpendriveObject
{
  friend class COpendriveRoadSegment;
  private:
    COpendriveRoadSegment *segment;
    int id; ///< Object's id.
    TOpendriveTrackPoint pose;
    TOpendriveObject object;
    std::string type; ///< Object's OpenDrive type.
    std::string name; ///< Object's name.
    object_type_t object_type;
    double scale_factor;
  protected:
    void load(objects::object_type &object_info,COpendriveRoadSegment *segment);
    void update_references(std::map<COpendriveRoadSegment *,COpendriveRoadSegment *> &segment_refs);
    void set_scale_factor(double scale);
  public:
    COpendriveObject();
    COpendriveObject(const COpendriveObject& object);
    double get_scale_factor(void);
    TOpendriveTrackPoint get_track_pose(void) const;
    TOpendriveWorldPoint get_world_pose(void) const;
    int get_id(void) const;
    std::string get_type(void) const;
    std::string get_name(void) const;
    bool is_box(void) const;
    bool is_cylinder(void) const;
    bool is_polygon(void) const;
    TOpendriveBox get_box_data(void) const;
    TOpendriveCylinder get_cylinder_data(void) const;
    TOpendrivePolygon get_polygon_data(void) const;
    friend std::ostream& operator<<(std::ostream& out, COpendriveObject &object);
    ~COpendriveObject();
};

#endif
