#ifndef _OPENDRIVE_ROAD_NODE_H
#define _OPENDRIVE_ROAD_NODE_H

#include <vector>
#include "opendrive_link.h"
#include "opendrive_lane.h"
#include "opendrive_road_segment.h"

class COpendriveLane;
class COpendriveRoadSegment;

class COpendriveRoadNode
{
  friend class COpendriveLane;
  friend class COpendriveRoadSegment;
  friend class COpendriveRoad;
  private:
    std::vector<COpendriveLink *> links;
    double resolution;
    double scale_factor;
    TOpendriveWorldPoint start_point;
    std::vector<COpendriveLane *> lanes;
    std::vector<COpendriveGeometry *> geometries;
    COpendriveRoadSegment *parent_segment;
    unsigned int index;
  protected:
    COpendriveRoadNode();
    COpendriveRoadNode(const COpendriveRoadNode &object);
    void load(const planView::geometry_type &geom_info,COpendriveLane *lane);
    void add_link(COpendriveRoadNode *node,opendrive_mark_t mark);
    void set_resolution(double res);
    void set_scale_factor(double scale);
    void set_index(unsigned int index);
    void set_parent_segment(COpendriveRoadSegment *segment);
    void add_lane(const planView::geometry_type &geom_info,COpendriveLane *lane);
    void update_references(std::map<COpendriveRoadNode *,COpendriveRoadNode *> refs);
    void update_references(std::map<COpendriveRoadSegment *,COpendriveRoadSegment *> refs);
    void update_references(std::map<COpendriveLane *,COpendriveLane *> refs);
  public:
    double get_resolution(void) const;
    double get_scale_factor(void) const;
    unsigned int get_index(void) const;
    const COpendriveRoadSegment& get_parent_segment(void) const;
    TOpendriveWorldPoint get_start_pose(void) const;
    unsigned int get_num_links(void) const;
    const COpendriveLink &get_link(unsigned int index) const;
    const COpendriveLink &get_link_ending_at(unsigned int node_index) const;
    unsigned int get_num_lanes(void) const; 
    const COpendriveLane &get_lane(unsigned int index) const;
    unsigned int get_num_geometries(void) const;
    const COpendriveGeometry &get_geometry(unsigned int index) const;
    unsigned int get_closest_link(double x, double y,double heading,double angle_tol=0.1) const;
    double get_closest_point(double x, double y,double heading,TOpendriveWorldPoint &closest_point,double angle_tol=0.1) const;
    void get_closest_points(double x, double y,double heading,std::vector<TOpendriveWorldPoint> &closest_points,std::vector<double> &closest_lengths,double dist_tol,double angle_tol=0.1) const;
    friend std::ostream& operator<<(std::ostream& out, COpendriveRoadNode &node);
    ~COpendriveRoadNode();
};

#endif
