#include "opendrive_spiral.h"

COpendriveSpiral::COpendriveSpiral()
{
  this->start_curvature=0.0;
  this->end_curvature=0.0;
}

COpendriveSpiral::COpendriveSpiral(const COpendriveSpiral &object) : COpendriveGeometry(object)
{
  this->start_curvature=object.start_curvature;
  this->end_curvature=object.end_curvature;
}

bool COpendriveSpiral::transform_local_pose(TOpendriveTrackPoint &track,TOpendriveLocalPoint &local) const
{
  return true;
}

void COpendriveSpiral::print(std::ostream &out)
{
  double start_curvature,end_curvature;

  COpendriveGeometry::print(out);
  this->get_curvature(start_curvature,end_curvature);
  out << "          start_curvature = " << start_curvature << ", end_curvature = " << end_curvature << std::endl;
}

void COpendriveSpiral::load_params(const planView::geometry_type &geometry_info)
{
  this->start_curvature = (geometry_info.spiral().get().curvStart().present() ? geometry_info.spiral().get().curvStart().get() : 0.0);
  this->end_curvature = (geometry_info.spiral().get().curvEnd().present() ? geometry_info.spiral().get().curvEnd().get() : 0.0);
}

std::string COpendriveSpiral::get_name(void)
{
  return std::string("spiral");
}

COpendriveGeometry *COpendriveSpiral::clone(void)
{
  COpendriveSpiral *new_spiral=new COpendriveSpiral(*this);

  return new_spiral;
}

void COpendriveSpiral::get_curvature(double &start,double &end)
{
  start=this->start_curvature*this->scale_factor;
  end=this->end_curvature*this->scale_factor;
}

void COpendriveSpiral::operator=(const COpendriveSpiral &object)
{
  COpendriveGeometry::operator=(object);
  this->start_curvature=object.start_curvature;
  this->end_curvature=object.end_curvature;
}

COpendriveSpiral::~COpendriveSpiral()
{
  this->start_curvature=0.0;
  this->end_curvature=0.0;
}

